

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

Represents a blob stream object.
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Ggit.Objects.BlobOutputStream
    ( 

-- * Exported types
    BlobOutputStream(..)                    ,
    IsBlobOutputStream                      ,
    toBlobOutputStream                      ,
    noBlobOutputStream                      ,


 -- * Methods
-- ** getId #method:getId#

#if ENABLE_OVERLOADING
    BlobOutputStreamGetIdMethodInfo         ,
#endif
    blobOutputStreamGetId                   ,




 -- * Properties
-- ** repository #attr:repository#
{- | /No description available in the introspection data./
-}
#if ENABLE_OVERLOADING
    BlobOutputStreamRepositoryPropertyInfo  ,
#endif
#if ENABLE_OVERLOADING
    blobOutputStreamRepository              ,
#endif
    constructBlobOutputStreamRepository     ,




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL

import qualified GI.GObject.Objects.Object as GObject.Object
import {-# SOURCE #-} qualified GI.Ggit.Objects.Repository as Ggit.Repository
import {-# SOURCE #-} qualified GI.Ggit.Structs.OId as Ggit.OId
import qualified GI.Gio.Objects.OutputStream as Gio.OutputStream

-- | Memory-managed wrapper type.
newtype BlobOutputStream = BlobOutputStream (ManagedPtr BlobOutputStream)
foreign import ccall "ggit_blob_output_stream_get_type"
    c_ggit_blob_output_stream_get_type :: IO GType

instance GObject BlobOutputStream where
    gobjectType = c_ggit_blob_output_stream_get_type
    

-- | Type class for types which can be safely cast to `BlobOutputStream`, for instance with `toBlobOutputStream`.
class (GObject o, O.IsDescendantOf BlobOutputStream o) => IsBlobOutputStream o
instance (GObject o, O.IsDescendantOf BlobOutputStream o) => IsBlobOutputStream o

instance O.HasParentTypes BlobOutputStream
type instance O.ParentTypes BlobOutputStream = '[Gio.OutputStream.OutputStream, GObject.Object.Object]

-- | Cast to `BlobOutputStream`, for types for which this is known to be safe. For general casts, use `Data.GI.Base.ManagedPtr.castTo`.
toBlobOutputStream :: (MonadIO m, IsBlobOutputStream o) => o -> m BlobOutputStream
toBlobOutputStream = liftIO . unsafeCastTo BlobOutputStream

-- | A convenience alias for `Nothing` :: `Maybe` `BlobOutputStream`.
noBlobOutputStream :: Maybe BlobOutputStream
noBlobOutputStream = Nothing

#if ENABLE_OVERLOADING
type family ResolveBlobOutputStreamMethod (t :: Symbol) (o :: *) :: * where
    ResolveBlobOutputStreamMethod "bindProperty" o = GObject.Object.ObjectBindPropertyMethodInfo
    ResolveBlobOutputStreamMethod "bindPropertyFull" o = GObject.Object.ObjectBindPropertyFullMethodInfo
    ResolveBlobOutputStreamMethod "clearPending" o = Gio.OutputStream.OutputStreamClearPendingMethodInfo
    ResolveBlobOutputStreamMethod "close" o = Gio.OutputStream.OutputStreamCloseMethodInfo
    ResolveBlobOutputStreamMethod "closeAsync" o = Gio.OutputStream.OutputStreamCloseAsyncMethodInfo
    ResolveBlobOutputStreamMethod "closeFinish" o = Gio.OutputStream.OutputStreamCloseFinishMethodInfo
    ResolveBlobOutputStreamMethod "flush" o = Gio.OutputStream.OutputStreamFlushMethodInfo
    ResolveBlobOutputStreamMethod "flushAsync" o = Gio.OutputStream.OutputStreamFlushAsyncMethodInfo
    ResolveBlobOutputStreamMethod "flushFinish" o = Gio.OutputStream.OutputStreamFlushFinishMethodInfo
    ResolveBlobOutputStreamMethod "forceFloating" o = GObject.Object.ObjectForceFloatingMethodInfo
    ResolveBlobOutputStreamMethod "freezeNotify" o = GObject.Object.ObjectFreezeNotifyMethodInfo
    ResolveBlobOutputStreamMethod "getv" o = GObject.Object.ObjectGetvMethodInfo
    ResolveBlobOutputStreamMethod "hasPending" o = Gio.OutputStream.OutputStreamHasPendingMethodInfo
    ResolveBlobOutputStreamMethod "isClosed" o = Gio.OutputStream.OutputStreamIsClosedMethodInfo
    ResolveBlobOutputStreamMethod "isClosing" o = Gio.OutputStream.OutputStreamIsClosingMethodInfo
    ResolveBlobOutputStreamMethod "isFloating" o = GObject.Object.ObjectIsFloatingMethodInfo
    ResolveBlobOutputStreamMethod "notify" o = GObject.Object.ObjectNotifyMethodInfo
    ResolveBlobOutputStreamMethod "notifyByPspec" o = GObject.Object.ObjectNotifyByPspecMethodInfo
    ResolveBlobOutputStreamMethod "ref" o = GObject.Object.ObjectRefMethodInfo
    ResolveBlobOutputStreamMethod "refSink" o = GObject.Object.ObjectRefSinkMethodInfo
    ResolveBlobOutputStreamMethod "runDispose" o = GObject.Object.ObjectRunDisposeMethodInfo
    ResolveBlobOutputStreamMethod "splice" o = Gio.OutputStream.OutputStreamSpliceMethodInfo
    ResolveBlobOutputStreamMethod "spliceAsync" o = Gio.OutputStream.OutputStreamSpliceAsyncMethodInfo
    ResolveBlobOutputStreamMethod "spliceFinish" o = Gio.OutputStream.OutputStreamSpliceFinishMethodInfo
    ResolveBlobOutputStreamMethod "stealData" o = GObject.Object.ObjectStealDataMethodInfo
    ResolveBlobOutputStreamMethod "stealQdata" o = GObject.Object.ObjectStealQdataMethodInfo
    ResolveBlobOutputStreamMethod "thawNotify" o = GObject.Object.ObjectThawNotifyMethodInfo
    ResolveBlobOutputStreamMethod "unref" o = GObject.Object.ObjectUnrefMethodInfo
    ResolveBlobOutputStreamMethod "watchClosure" o = GObject.Object.ObjectWatchClosureMethodInfo
    ResolveBlobOutputStreamMethod "write" o = Gio.OutputStream.OutputStreamWriteMethodInfo
    ResolveBlobOutputStreamMethod "writeAll" o = Gio.OutputStream.OutputStreamWriteAllMethodInfo
    ResolveBlobOutputStreamMethod "writeAllAsync" o = Gio.OutputStream.OutputStreamWriteAllAsyncMethodInfo
    ResolveBlobOutputStreamMethod "writeAllFinish" o = Gio.OutputStream.OutputStreamWriteAllFinishMethodInfo
    ResolveBlobOutputStreamMethod "writeAsync" o = Gio.OutputStream.OutputStreamWriteAsyncMethodInfo
    ResolveBlobOutputStreamMethod "writeBytes" o = Gio.OutputStream.OutputStreamWriteBytesMethodInfo
    ResolveBlobOutputStreamMethod "writeBytesAsync" o = Gio.OutputStream.OutputStreamWriteBytesAsyncMethodInfo
    ResolveBlobOutputStreamMethod "writeBytesFinish" o = Gio.OutputStream.OutputStreamWriteBytesFinishMethodInfo
    ResolveBlobOutputStreamMethod "writeFinish" o = Gio.OutputStream.OutputStreamWriteFinishMethodInfo
    ResolveBlobOutputStreamMethod "writev" o = Gio.OutputStream.OutputStreamWritevMethodInfo
    ResolveBlobOutputStreamMethod "writevAll" o = Gio.OutputStream.OutputStreamWritevAllMethodInfo
    ResolveBlobOutputStreamMethod "writevAllAsync" o = Gio.OutputStream.OutputStreamWritevAllAsyncMethodInfo
    ResolveBlobOutputStreamMethod "writevAllFinish" o = Gio.OutputStream.OutputStreamWritevAllFinishMethodInfo
    ResolveBlobOutputStreamMethod "writevAsync" o = Gio.OutputStream.OutputStreamWritevAsyncMethodInfo
    ResolveBlobOutputStreamMethod "writevFinish" o = Gio.OutputStream.OutputStreamWritevFinishMethodInfo
    ResolveBlobOutputStreamMethod "getData" o = GObject.Object.ObjectGetDataMethodInfo
    ResolveBlobOutputStreamMethod "getId" o = BlobOutputStreamGetIdMethodInfo
    ResolveBlobOutputStreamMethod "getProperty" o = GObject.Object.ObjectGetPropertyMethodInfo
    ResolveBlobOutputStreamMethod "getQdata" o = GObject.Object.ObjectGetQdataMethodInfo
    ResolveBlobOutputStreamMethod "setData" o = GObject.Object.ObjectSetDataMethodInfo
    ResolveBlobOutputStreamMethod "setPending" o = Gio.OutputStream.OutputStreamSetPendingMethodInfo
    ResolveBlobOutputStreamMethod "setProperty" o = GObject.Object.ObjectSetPropertyMethodInfo
    ResolveBlobOutputStreamMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveBlobOutputStreamMethod t BlobOutputStream, O.MethodInfo info BlobOutputStream p) => OL.IsLabel t (BlobOutputStream -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif

-- VVV Prop "repository"
   -- Type: TInterface (Name {namespace = "Ggit", name = "Repository"})
   -- Flags: [PropertyWritable,PropertyConstructOnly]
   -- Nullable: (Nothing,Nothing)

{- |
Construct a `GValueConstruct` with valid value for the “@repository@” property. This is rarely needed directly, but it is used by `Data.GI.Base.Constructible.new`.
-}
constructBlobOutputStreamRepository :: (IsBlobOutputStream o, Ggit.Repository.IsRepository a) => a -> IO (GValueConstruct o)
constructBlobOutputStreamRepository val = B.Properties.constructObjectPropertyObject "repository" (Just val)

#if ENABLE_OVERLOADING
data BlobOutputStreamRepositoryPropertyInfo
instance AttrInfo BlobOutputStreamRepositoryPropertyInfo where
    type AttrAllowedOps BlobOutputStreamRepositoryPropertyInfo = '[ 'AttrConstruct, 'AttrClear]
    type AttrSetTypeConstraint BlobOutputStreamRepositoryPropertyInfo = Ggit.Repository.IsRepository
    type AttrBaseTypeConstraint BlobOutputStreamRepositoryPropertyInfo = IsBlobOutputStream
    type AttrGetType BlobOutputStreamRepositoryPropertyInfo = ()
    type AttrLabel BlobOutputStreamRepositoryPropertyInfo = "repository"
    type AttrOrigin BlobOutputStreamRepositoryPropertyInfo = BlobOutputStream
    attrGet _ = undefined
    attrSet _ = undefined
    attrConstruct _ = constructBlobOutputStreamRepository
    attrClear _ = undefined
#endif

#if ENABLE_OVERLOADING
instance O.HasAttributeList BlobOutputStream
type instance O.AttributeList BlobOutputStream = BlobOutputStreamAttributeList
type BlobOutputStreamAttributeList = ('[ '("repository", BlobOutputStreamRepositoryPropertyInfo)] :: [(Symbol, *)])
#endif

#if ENABLE_OVERLOADING
blobOutputStreamRepository :: AttrLabelProxy "repository"
blobOutputStreamRepository = AttrLabelProxy

#endif

#if ENABLE_OVERLOADING
type instance O.SignalList BlobOutputStream = BlobOutputStreamSignalList
type BlobOutputStreamSignalList = ('[ '("notify", GObject.Object.ObjectNotifySignalInfo)] :: [(Symbol, *)])

#endif

-- method BlobOutputStream::get_id
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "stream", argType = TInterface (Name {namespace = "Ggit", name = "BlobOutputStream"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GgitBlobOutputStream.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TInterface (Name {namespace = "Ggit", name = "OId"}))
-- throws : True
-- Skip return : False

foreign import ccall "ggit_blob_output_stream_get_id" ggit_blob_output_stream_get_id :: 
    Ptr BlobOutputStream ->                 -- stream : TInterface (Name {namespace = "Ggit", name = "BlobOutputStream"})
    Ptr (Ptr GError) ->                     -- error
    IO (Ptr Ggit.OId.OId)

{- |
Get the id of the written blob. The blob id is only available after the
stream has been properly closed. If an error occurred while writing the blob,
the 'Nothing' is returned and /@error@/ is set accordingly.
-}
blobOutputStreamGetId ::
    (B.CallStack.HasCallStack, MonadIO m, IsBlobOutputStream a) =>
    a
    {- ^ /@stream@/: a 'GI.Ggit.Objects.BlobOutputStream.BlobOutputStream'. -}
    -> m (Maybe Ggit.OId.OId)
    {- ^ __Returns:__ a 'GI.Ggit.Structs.OId.OId' or 'Nothing'. /(Can throw 'Data.GI.Base.GError.GError')/ -}
blobOutputStreamGetId stream = liftIO $ do
    stream' <- unsafeManagedPtrCastPtr stream
    onException (do
        result <- propagateGError $ ggit_blob_output_stream_get_id stream'
        maybeResult <- convertIfNonNull result $ \result' -> do
            result'' <- (wrapBoxed Ggit.OId.OId) result'
            return result''
        touchManagedPtr stream
        return maybeResult
     ) (do
        return ()
     )

#if ENABLE_OVERLOADING
data BlobOutputStreamGetIdMethodInfo
instance (signature ~ (m (Maybe Ggit.OId.OId)), MonadIO m, IsBlobOutputStream a) => O.MethodInfo BlobOutputStreamGetIdMethodInfo a signature where
    overloadedMethod _ = blobOutputStreamGetId

#endif


